# -*- coding: utf-8 -*-
"""
CONVERT_SEG2TOSU.py
@date: 2021/11/25
@author: p.bottelin
@affilitation: adrgt

Description: cf. READ_SEG2.pdf

Usage: Placer le script CONVERT_SEG2TOSU directement dans le dossier contenant les .DAT à convertir
Ensuite, lancer gitbash (ou autre terminal) dans ce même dossier
Activer l'environnement qui va bien (conda activate vib_prod par exemple,
ou conda activate pymonitrok_prod, ou similaire)
ensuite, taper python CONVERT_SEG2TOSU.pymonitrok_prod
Le script s'exécute, et affiche à l'écran les fichiers convertis.

v00: problème de lecture du pre-trigger delay constaté par nico moreau le 2021/11/24
v01: lecture de spacing et de pre-trigger delay depuis le header du .DAT et écriture dans le header
du .SU
v02: essai d'écriture de group_coordinate et source_coordinate dans le header du .SU mais probleme
car ne tolère pas les float (integer required)
"""
############################# IMPORTS ############################
import obspy
import glob
import os
import numpy as np
from obspy.io.segy.core import SEGYTraceHeader, SEGYBinaryFileHeader
from obspy.io.segy.core import _read_segy
from obspy.core import AttribDict

########################### PARAMETERS ##############################

## Parameter 1 : pattern, c'est la "gueule" des fichiers à convertir, en chemin relatif par rapport au script
## je conseille de mettre le script directement dans le dossier contenant les .DAT à convertir
## dans ce cas, pattern = "*.DAT" en général
## note: * est un wildcard, ça veut dire "quelle que soit la chaîne de caractères"
## donc "*.DAT" va traiter tous les fichiers finissant en ".DAT"
pattern = "./EXAMPLE/*.DAT" 




############################## MAIN ###########################


##FILE LIST
pattern_dat = os.path.normpath(pattern)
filelist_dat = glob.glob(pattern_dat)
print(">>> Voici la liste des fichiers a convertir:")
print(">>> Here is the list of files to convert:")
print(filelist_dat)


##LOOP
print(">>> Début de la conversion .DAT > .SU")
for filename in filelist_dat:
    # """ 1. read .dat file """
    st_seg2 = obspy.read(filename)
    ## catch spacing field
    spacing = float(st_seg2.stats['seg2']['NOTE'][0].split('BASE_INTERVAL')[1])
    print(st_seg2[0].stats)
    print('Please check : spacing = ' + str(spacing) + ' m')
    ## put distance field
    for idx, tr_seg2 in enumerate(st_seg2):
        tr_seg2.stats.distance = idx * spacing
    ## change trace format
    for idx, tr_seg2 in enumerate(st_seg2):
        tr_seg2.data = np.require(tr_seg2.data, dtype=np.float32)
    ## writes to disk
    newfname = filename.split(".DAT")[0] + ".SU"
    st_seg2.write(newfname, format='SU')
    
    
    
    ## reopen SEG2 and SU simultaneously and rewrite delay time trace by trace
    st_su = obspy.read(newfname)
    ##store attribdict delay
    for idx, (tr_seg2, tr_su) in enumerate(zip(st_seg2, st_su)):
        print(idx, tr_seg2, tr_su)
        ##change pretrig delay
        delay_trace = int(np.round(1000*float(tr_seg2.stats.seg2['DELAY'])))
        print('Please check : delay = ' + str(delay_trace) + ' ms')
        tr_su.stats.su.trace_header['delay_recording_time'] = delay_trace ## ms
        ##change header
        tr_su.stats.su.trace_header['trace_sequence_number_within_line'] = idx
        tr_su.stats.su.trace_header['trace_sequence_number_within_segy_file'] = idx
        tr_su.stats.su.trace_header['original_field_record_number'] = idx
        tr_su.stats.su.trace_header['trace_number_within_the_original_field_record'] = idx
        tr_su.stats.su.trace_header['energy_source_point_number'] = idx
        tr_su.stats.su.trace_header['ensemble_number'] = idx
        tr_su.stats.su.trace_header['trace_number_within_the_ensemble'] = idx
        tr_su.stats.su.trace_header['trace_identification_code'] = idx
        ##Source position
        tr_su.stats.su.trace_header['counit'] = 1
        tr_su.stats.su.trace_header['source_coordinate_x'] = int(10*float(tr_seg2.stats.seg2['SOURCE_LOCATION']))
        tr_su.stats.su.trace_header['source_coordinate_y'] = int(float(0.0)) # 0 m along y
        ##Geophone position
        tr_su.stats.su.trace_header['group_coordinate_x'] = int(10* float(spacing) * float(idx))
        tr_su.stats.su.trace_header['group_coordinate_y'] = int(float(0.0)) # 0 m along y
        tr_su.stats.su.trace_header['shotpoint_number'] = idx
        tr_su.stats.su.trace_header['device_trace_identifier'] = idx
        tr_su.stats.su.trace_header['name'] = 'test'
        tr_su.stats.su.trace_header['raw record'] = 'test'
        ##change trace format
        tr_su.data = np.require(tr_su.data, dtype=np.float32)
        print(tr_su.stats.su.trace_header)
        tr_su.stats.su.trace_header.update()
        
    ##writes to disk
    newfname = filename.split(".DAT")[0] + ".SU"
    st_su.write(newfname, format='SU')


print(">>> FIN DE LA CONVERSION")
print(">>> END OF CONVERSION")






